<?php

/**
 * Class FLWoocommerce
 */
final class FLWoocommerce {
	/**
	 * Initialize
	 */
	public static function init( ){
		if( ! class_exists( 'woocommerce' ) ) {
			return;
		}

		add_action( 'wp_enqueue_scripts', 'FLWoocommerce::enqueue_scripts' );

		if ( 'enabled' == FLTheme::get_setting( 'fl-woo-css' ) ) {
			remove_action( 'woocommerce_before_shop_loop_item_title', 'woocommerce_template_loop_product_thumbnail', 10 );
			remove_action( 'woocommerce_before_main_content', 'woocommerce_breadcrumb', 20, 0 );

			add_action( 'woocommerce_before_shop_loop_item_title', 'FLWoocommerce::template_loop_product_thumbnail', 10 );
			add_action( 'woocommerce_before_shop_loop_item_title', 'FLWoocommerce::show_product_loop_new_badge', 30 );    // The new badge function
			add_action( 'woocommerce_after_shop_loop_item_title', 'woocommerce_template_single_excerpt', 5 );

			add_filter( 'loop_shop_per_page', 'FLWoocommerce::loop_shop_per_page', 20 );
			add_filter( 'woocommerce_short_description', 'FLWoocommerce::woocommerce_short_description' );
			add_filter( 'woocommerce_loop_add_to_cart_link', 'FLWoocommerce::loop_show_more_button', 20, 2 );
			add_filter( 'woocommerce_product_single_add_to_cart_text', 'FLWoocommerce::cart_button_text' );    // 2.1 +
			add_filter( 'woocommerce_product_tabs', 'FLWoocommerce::rename_tabs', 98 );
			add_filter( 'woocommerce_catalog_orderby', 'FLWoocommerce::catalog_ordering', 98 );
			add_filter( 'woocommerce_create_pages', 'FLWoocommerce::create_pages' );

			add_action( 'fl_woocommerce_before_product_thumbnail', 'FLWoocommerce::fl_woocommerce_before_product_thumbnail' );
			add_action( 'woocommerce_after_shop_loop_item_title', 'FLWoocommerce::woocommerce_after_shop_loop_item_title' );
		}

		add_filter( 'woocommerce_before_main_content', 'FLWoocommerce::breadcrumbs', 10 );
		add_filter( 'wp_nav_menu_items', 'FLWoocommerce::cart_menu_link', 10, 2 );

		if ( is_rtl() ) {
			add_filter( 'woocommerce_pagination_args', 'FLWoocommerce::rtl_pagination_links' );
			add_filter( 'woocommerce_comment_pagination_args', 'FLWoocommerce::rtl_pagination_links' );
		}
	}

	#region Filters and actions

	/**
	 * Enqueue WooCommerce styles and scripts
	 */
	public static function enqueue_scripts() {
		global $woocommerce;

		$suffix      = defined( 'SCRIPT_DEBUG' ) && SCRIPT_DEBUG ? '' : '.min';
		$lightbox_en = get_option( 'woocommerce_enable_lightbox' ) == 'yes' ? true : false;

		wp_enqueue_style('slick-slider', WIPI_THEME_URL . '/css/slick.css', false, '1.6.0');
		wp_enqueue_style('slick-slider-theme', WIPI_THEME_URL . '/css/slick-theme.css', array('slick-slider'), '1.6.0');
		wp_enqueue_script('slick-slider', WIPI_THEME_URL . '/js/slick.min.js', array( 'jquery' ), '1.6.0', true);
		wp_enqueue_script('wipi-woocommerce', WIPI_THEME_URL . '/js/woocommerce.js', array( 'jquery', 'slick-slider' ), WIPI_THEME_VERSION, true);
		wp_localize_script('wipi-woocommerce', 'wipiConfig', array(
			'isRtl' => is_rtl(),
		));

		if( is_rtl() ) {
			// Woocommerce RTL
			wp_dequeue_style( 'woocommerce-layout' );
			wp_dequeue_style( 'woocommerce-smallscreen' );
			wp_dequeue_style( 'woocommerce-general' );
			wp_enqueue_style( 'woocommerce-layout-rtl', WIPI_THEME_URL . '/css/woocommerce-layout.rtl.css', array(), $woocommerce->version );
			wp_enqueue_style( 'woocommerce-smallscreen-rtl', WIPI_THEME_URL . '/css/woocommerce-smallscreen.rtl.css', array(), $woocommerce->version, 'only screen and (max-width: 768px)' );
			wp_enqueue_style( 'woocommerce-general-rtl', WIPI_THEME_URL . '/css/woocommerce.rtl.css', array(), $woocommerce->version );
		}

		if ( $lightbox_en ) {
			wp_enqueue_script( 'prettyPhoto', $woocommerce->plugin_url() . '/assets/js/prettyPhoto/jquery.prettyPhoto' . $suffix . '.js', array( 'jquery' ), $woocommerce->version, true );
			wp_enqueue_script( 'prettyPhoto-init', $woocommerce->plugin_url() . '/assets/js/prettyPhoto/jquery.prettyPhoto.init' . $suffix . '.js', array( 'jquery' ), $woocommerce->version, true );
			wp_enqueue_style( 'woocommerce_prettyPhoto_css', $woocommerce->plugin_url() . '/assets/css/prettyPhoto.css' );
		}
	}

	public static function breadcrumbs() {
		// any non-woocommerce pages handle the breadcrumbs already
		if ( apply_filters( 'wipi_disable_woocommerce_breadcrumbs', false ) || ! is_woocommerce() )
			return;

		return FLBreadcrumbs::breadcrumbs();
	}

	/**
	 *
	 */
	public static function template_loop_product_thumbnail() {
		do_action( 'fl_woocommerce_before_product_thumbnail' );
		echo self::get_product_thumbnail();
		do_action( 'fl_woocommerce_after_product_thumbnail' );
	}

	public static function rtl_pagination_links( $args ) {
		$args['prev_text'] = '&rarr;';
		$args['next_text'] = '&larr;';
		return $args;
	}


	/**
	 * @param string $size
	 * @param int $placeholder_width
	 * @param int $placeholder_height
	 *
	 * @return string
	 */
	public static function get_product_thumbnail( $size = 'shop_catalog', $placeholder_width = 0, $placeholder_height = 0  ) {
		global $post, $product;

		$attachment_ids = $product->get_gallery_attachment_ids();
		$output = '<div class="imagewrapper" ';

		if ( has_post_thumbnail() ) {
			$output .= 'style="background-image: url(' . wp_get_attachment_url( get_post_thumbnail_id( $post->ID, $size )) .');">';

			if(count($attachment_ids)) {
				$output .= '<div class="imagewrapper-hover" style="background-image: url(' . wp_get_attachment_url( $attachment_ids[0], "shop_single" ) . '"></div>';

			}
		} else {
			$output .= ">";
		}

		$output .= '</div>';
		return $output;
	}


	/**
	 * Add show more button to products in loop
	 * @param $link
	 * @param $product
	 *
	 * @return string
	 */
	public static function loop_show_more_button($link, $product) {
		$link .= '<a href="'. get_permalink( $product->id ) .'" class="show_more"><i class="fa fa-fw fa-search-plus"></i> ' . __( 'Learn more', 'wipi' ) . '</a>';

		$attachment_ids = $product->get_gallery_attachment_ids();
		$photos = array();
		if ( has_post_thumbnail() ) {
			$photos[] = wp_get_attachment_url( get_post_thumbnail_id( $product->ID ));
		}
		foreach( $attachment_ids as $att ) {
			$photos[] = wp_get_attachment_url( $att );
		}

		for( $i = 0; $i < count( $photos ); ++$i ) {
			$link .= '<a href="' . $photos[$i] . '" class="quick_view zoom first '. ( $i > 0 ? 'hidden' : '' ) .'" data-rel="prettyPhoto[product-'.$product->id.']"><i class="fa fa-fw fa-picture-o"></i> ' . __( 'Quick View', 'wipi' ) . '</a>';
		}
		return $link;
	}


	/**
	 * Add "New" badge for products in the loop
	 */
	public static function show_product_loop_new_badge() {
		$postdate      = get_the_time( 'Y-m-d' );            // Post date
		$postdatestamp = strtotime( $postdate );            // Timestamped post date
		$newness       = get_option( 'wc_nb_newness' );    // Newness in days as defined by option
		if ( ( time() - ( 60 * 60 * 24 * $newness ) ) < $postdatestamp ) { // If the product was published within the newness time frame display the new badge
			echo '<span class="wc-new-badge">' . __( 'New', 'wipi' ) . '</span>';
		}
	}


	/**
	 * Translate cart button text
	 * @return string
	 */
	public static function cart_button_text() {
		global $product;

		$product_type = $product->product_type;

		switch ( $product_type ) {
			case 'external':
				return __( 'Buy product', 'wipi' );
				break;
			case 'grouped':
				return __( 'View products', 'wipi' );
				break;
			case 'simple':
				return __( 'Add to cart', 'wipi' );
				break;
			case 'variable':
				return __( 'Select options', 'wipi' );
				break;
			default:
				return __( 'Read more', 'wipi' );
		}
	}


	/**
	 * Translate tab titles
	 * @param $tabs
	 *
	 * @return mixed
	 */
	public static function rename_tabs( $tabs ) {
		global $product, $post;
		if ( $post->post_content ) {
			if(!empty($tabs['description']['title'])) { $tabs['description']['title'] = __( 'Description', 'wipi' ); }		// Rename the description tab
			if(!empty($tabs['reviews']['title'])) { $tabs['reviews']['title'] = __( 'Reviews', 'wipi' ); }				// Rename the reviews tab
			if(!empty($tabs['additional_information']['title'])) { $tabs['additional_information']['title'] = __( 'Product Data', 'wipi' ); }	// Rename the additional information tab
		}
		return $tabs;
	}


	/**
	 * Translate ordering menu
	 * @param $orderby
	 *
	 * @return mixed
	 */
	public static function catalog_ordering( $orderby ) {
		$orderby['menu_order'] = __( 'Default sorting', 'wipi' );
		$orderby['popularity'] = __( 'Sort by popularity', 'wipi' );
		$orderby['rating'] = __( 'Sort by average rating', 'wipi' );
		$orderby['date'] = __( 'Sort by newness', 'wipi' );
		$orderby['price'] = __( 'Sort by price: low to high', 'wipi' );
		$orderby['price-desc'] = __( 'Sort by price: high to low', 'wipi' );

		return $orderby;
	}


	/**
	 * Translate page titles
	 * @param $pages
	 *
	 * @return mixed
	 */
	public static function create_pages ($pages) {
		$pages['shop'] = array(
			'name'    => 'shop',
			'title'   => _x( 'Shop', 'Page title', 'wipi' ),
			'content' => ''
		);
		$pages['cart'] = array(
			'name'    => 'cart',
			'title'   => _x( 'Cart', 'Page title', 'wipi' ),
			'content' => '[' . apply_filters( 'woocommerce_cart_shortcode_tag', 'woocommerce_cart' ) . ']'
		);
		$pages['checkout'] = array(
			'name'    => 'checkout',
			'title'   => _x( 'Checkout', 'Page title', 'wipi' ),
			'content' => '[' . apply_filters( 'woocommerce_checkout_shortcode_tag', 'woocommerce_checkout' ) . ']'
		);
		$pages['myaccount'] = array(
			'name'    => 'my-account',
			'title'   => _x( 'My Account', 'Page title', 'wipi' ),
			'content' => '[' . apply_filters( 'woocommerce_my_account_shortcode_tag', 'woocommerce_my_account' ) . ']'
		);
		return $pages;
	}


	public static function woocommerce_short_description( $post_excerpt ) {
		if ( 'visible' == FLTheme::get_setting( 'fl-woo-product-description-display' ) ) {
			return $post_excerpt;
		}
	}


	/**
	 * Specify columns for loop
	 * @param $cols
	 *
	 * @return int
	 */
	public static function loop_shop_per_page($cols) {
		return 9;
	}


	/**
	 * Add cart menu to main navigation
	 * @param $items
	 * @param $args
	 *
	 * @return string
	 */
	public static function cart_menu_link( $items, $args ) {
		global $woocommerce;

		if( 'header' != $args->theme_location || 'visible' != FLTheme::get_setting( 'fl-show-woocommerce-cart-link' ) ) {
			return $items;
		}

		$count = $woocommerce->cart->get_cart_contents_count();
		$text = __( 'Shopping Cart', 'wipi' );
		$url = wc_get_cart_url();
		$menu_item_id = 0;

		$cart_items_text = apply_filters( 'woocommerce_shopping_cart_link_items', sprintf(
			_x( '(%s)', 'Woocommerce cart items in navigation bar', 'wipi' ),
			$count
		), $count );

		if($count <= 0) {
			$cart_items_text = "";
		}


		$items .= "<li id=\"menu-item-{$menu_item_id}\" class=\"menu-item menu-item-type-woocommerce_cart menu-item-woocommerce_cart menu-item-{$menu_item_id}\">{$args->before}<a href=\"{$url}\">{$args->link_before}{$text} {$cart_items_text}{$args->link_after}</a>{$args->after}</li>";

		return $items;
	}

	public static function fl_woocommerce_before_product_thumbnail() {
		echo '<div class="product-wrapper">';
	}
	public static function woocommerce_after_shop_loop_item_title() {
		echo '</div>';
	}

	#endregion

	#region Utility functions

	/**
	 * Check if WooCommerce is available
	 * @return bool
	 */
	public static function woocommerce_available() {
		return class_exists('woocommerce');
	}

	/**
	 * Check if the current page is a WooCommerce page
	 * @return bool
	 */
	public static function is_woocommerce_page() {
		return self::woocommerce_available() && ( is_woocommerce() || is_product() || is_product_taxonomy() || is_product_category() || is_product_tag() || is_shop() || is_cart() ||  is_checkout() || is_account_page() );
	}

	#endregion
}
